@@ -103,7 +103,9 @@ impl Primitive for Name<'_> {
     fn write(self, buf: &mut Vec<u8>) {
         buf.push(b'/');
         for &byte in self.0 {
-            if matches!(byte, b'!' ..= b'~') && byte != b'#' {
+            // Quite a few characters must be escaped in names, so we take the
+            // safe route and allow only ASCII letters and numbers.
+            if byte.is_ascii_alphanumeric() {
                 buf.push(byte);
             } else {
                 buf.push(b'#');
@@ -656,7 +658,8 @@ pub struct Stream<'a> {
 impl<'a> Stream<'a> {
     /// Start writing a stream.
     ///
-    /// Panics if the object writer is not indirect.
+    /// Panics if the object writer is not indirect or the stream length exceeds
+    /// `i32::MAX`.
     pub(crate) fn start(obj: Obj<'a>, data: &'a [u8]) -> Self {
         assert!(obj.indirect);
 
