@@ -1,19 +1,21 @@
 use super::*;
 
-/// Writer for a _document catalog_.
+/// Writer for a _document catalog dictionary_.
 ///
 /// This struct is created by [`PdfWriter::catalog`].
 pub struct Catalog<'a> {
-    dict: Dict<'a, IndirectGuard>,
+    dict: Dict<'a>,
 }
 
-impl<'a> Catalog<'a> {
-    pub(crate) fn start(obj: Obj<'a, IndirectGuard>) -> Self {
+impl<'a> Writer<'a> for Catalog<'a> {
+    fn start(obj: Obj<'a>) -> Self {
         let mut dict = obj.dict();
         dict.pair(Name(b"Type"), Name(b"Catalog"));
         Self { dict }
     }
+}
 
+impl<'a> Catalog<'a> {
     /// Write the `/Pages` attribute pointing to the root page tree.
     pub fn pages(&mut self, id: Ref) -> &mut Self {
         self.pair(Name(b"Pages"), id);
@@ -42,173 +44,31 @@ impl<'a> Catalog<'a> {
     }
 
     /// Write the `/Dests` attribute pointing to a
-    /// [named destinations dictionary](Destinations). (1.1+)
+    /// [named destinations dictionary](Destinations). PDF 1.1+.
     pub fn destinations(&mut self, id: Ref) -> &mut Self {
         self.pair(Name(b"Dests"), id);
         self
     }
 
-    /// Start writing the `/ViewerPreferences` dictionary. (1.2+)
+    /// Start writing the `/ViewerPreferences` dictionary. PDF 1.2+.
     pub fn viewer_preferences(&mut self) -> ViewerPreferences<'_> {
-        ViewerPreferences::new(self.key(Name(b"ViewerPreferences")))
+        self.insert(Name(b"ViewerPreferences")).start()
+    }
+
+    /// Write the `/PageLabels` attribute to specify the page labels. PDF 1.3+.
+    pub fn page_labels(&mut self) -> Dict<'_> {
+        self.insert(Name(b"PageLabels")).start()
+    }
+
+    /// Write the `/Lang` attribute to specify the language of the document as a
+    /// RFC 3066 language tag. PDF 1.4+.
+    pub fn lang(&mut self, lang: TextStr) -> &mut Self {
+        self.pair(Name(b"Lang"), lang);
+        self
     }
 }
 
-deref!('a, Catalog<'a> => Dict<'a, IndirectGuard>, dict);
-
-/// Writer for a _page tree_.
-///
-/// This struct is created by [`PdfWriter::pages`].
-pub struct Pages<'a> {
-    dict: Dict<'a, IndirectGuard>,
-}
-
-impl<'a> Pages<'a> {
-    pub(crate) fn start(obj: Obj<'a, IndirectGuard>) -> Self {
-        let mut dict = obj.dict();
-        dict.pair(Name(b"Type"), Name(b"Pages"));
-        Self { dict }
-    }
-
-    /// Write the `/Parent` attribute.
-    pub fn parent(&mut self, parent: Ref) -> &mut Self {
-        self.pair(Name(b"Parent"), parent);
-        self
-    }
-
-    /// Write the `/Kids` and `/Count` attributes.
-    pub fn kids(&mut self, kids: impl IntoIterator<Item = Ref>) -> &mut Self {
-        let len = self.key(Name(b"Kids")).array().typed().items(kids).len();
-        self.pair(Name(b"Count"), len);
-        self
-    }
-
-    /// Write the `/MediaBox` attribute.
-    pub fn media_box(&mut self, rect: Rect) -> &mut Self {
-        self.pair(Name(b"MediaBox"), rect);
-        self
-    }
-
-    /// Start writing the `/Resources` dictionary.
-    pub fn resources(&mut self) -> Resources<'_> {
-        Resources::new(self.key(Name(b"Resources")))
-    }
-}
-
-deref!('a, Pages<'a> => Dict<'a, IndirectGuard>, dict);
-
-/// Writer for a _page_.
-///
-/// This struct is created by [`PdfWriter::page`].
-pub struct Page<'a> {
-    dict: Dict<'a, IndirectGuard>,
-}
-
-impl<'a> Page<'a> {
-    pub(crate) fn start(obj: Obj<'a, IndirectGuard>) -> Self {
-        let mut dict = obj.dict();
-        dict.pair(Name(b"Type"), Name(b"Page"));
-        Self { dict }
-    }
-
-    /// Write the `/Parent` attribute.
-    pub fn parent(&mut self, parent: Ref) -> &mut Self {
-        self.pair(Name(b"Parent"), parent);
-        self
-    }
-
-    /// Write the `/MediaBox` attribute. This is the size of the physical medium
-    /// the page gets printed onto.
-    pub fn media_box(&mut self, rect: Rect) -> &mut Self {
-        self.pair(Name(b"MediaBox"), rect);
-        self
-    }
-
-    /// Write the `/CropBox` attribute. This is the size of the area within
-    /// which content is visible.
-    pub fn crop_box(&mut self, rect: Rect) -> &mut Self {
-        self.pair(Name(b"CropBox"), rect);
-        self
-    }
-
-    /// Write the `/BleedBox` attribute. This is the size of the area within
-    /// which content is visible in a print production environment. Most
-    /// production-aiding marks should be outside of this box. (1.3+)
-    pub fn bleed_box(&mut self, rect: Rect) -> &mut Self {
-        self.pair(Name(b"BleedBox"), rect);
-        self
-    }
-
-    /// Write the `/TrimBox` attribute. This is the size of the produced
-    /// document after trimming is applied. (1.3+)
-    pub fn trim_box(&mut self, rect: Rect) -> &mut Self {
-        self.pair(Name(b"TrimBox"), rect);
-        self
-    }
-
-    /// Write the `/ArtBox` attribute. This is the area that another program
-    /// importing this file should use. (1.3+)
-    pub fn art_box(&mut self, rect: Rect) -> &mut Self {
-        self.pair(Name(b"ArtBox"), rect);
-        self
-    }
-
-    /// Start writing the `/Resources` dictionary.
-    pub fn resources(&mut self) -> Resources<'_> {
-        Resources::new(self.key(Name(b"Resources")))
-    }
-
-    /// Write the `/Contents` attribute.
-    pub fn contents(&mut self, id: Ref) -> &mut Self {
-        self.pair(Name(b"Contents"), id);
-        self
-    }
-
-    /// Write the `/Dur` attribute. This is the amount of seconds the page
-    /// should be displayed before advancing to the next one. (1.1+)
-    pub fn duration(&mut self, seconds: f32) -> &mut Self {
-        self.pair(Name(b"Dur"), seconds);
-        self
-    }
-
-    /// Start writing the `/Trans` dictionary. This sets a transition effect for
-    /// advancing to the next page. (1.1+)
-    pub fn transition(&mut self) -> Transition<'_> {
-        Transition::new(self.key(Name(b"Trans")))
-    }
-
-    /// Start writing the `/Annots` (annotations) array.
-    pub fn annotations(&mut self) -> Annotations<'_> {
-        Annotations::start(self.key(Name(b"Annots")))
-    }
-}
-
-deref!('a, Page<'a> => Dict<'a, IndirectGuard>, dict);
-
-/// Writer for a _resource dictionary_.
-///
-/// This struct is created by [`Pages::resources`] and [`Page::resources`].
-pub struct Resources<'a> {
-    dict: Dict<'a>,
-}
-
-impl<'a> Resources<'a> {
-    pub(crate) fn new(obj: Obj<'a>) -> Self {
-        Self { dict: obj.dict() }
-    }
-
-    /// Start writing the `/XObject` dictionary.
-    pub fn x_objects(&mut self) -> TypedDict<'_, Ref> {
-        self.key(Name(b"XObject")).dict().typed()
-    }
-
-    /// Start writing the `/Font` dictionary.
-    pub fn fonts(&mut self) -> TypedDict<'_, Ref> {
-        self.key(Name(b"Font")).dict().typed()
-    }
-}
-
-deref!('a, Resources<'a> => Dict<'a>, dict);
+deref!('a, Catalog<'a> => Dict<'a>, dict);
 
 /// How the viewer should lay out the pages in the document.
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Hash)]
@@ -224,15 +84,15 @@ pub enum PageLayout {
     /// pages on the right (like in a left-bound book).
     TwoColumnRight,
     /// Only two pages are visible at a time, laid out with odd-numbered pages
-    /// on the left. (1.5+)
+    /// on the left. PDF 1.5+.
     TwoPageLeft,
     /// Only two pages are visible at a time, laid out with odd-numbered pages
-    /// on the right (like in a left-bound book). (1.5+)
+    /// on the right (like in a left-bound book). PDF 1.5+.
     TwoPageRight,
 }
 
 impl PageLayout {
-    fn to_name(self) -> Name<'static> {
+    pub(crate) fn to_name(self) -> Name<'static> {
         match self {
             Self::SinglePage => Name(b"SinglePage"),
             Self::OneColumn => Name(b"OneColumn"),
@@ -260,7 +120,7 @@ pub enum PageMode {
 }
 
 impl PageMode {
-    fn to_name(self) -> Name<'static> {
+    pub(crate) fn to_name(self) -> Name<'static> {
         match self {
             Self::UseNone => Name(b"UseNone"),
             Self::UseOutlines => Name(b"UseOutlines"),
@@ -270,20 +130,451 @@ impl PageMode {
     }
 }
 
+/// Writer for a _viewer preference dictionary_.
+///
+/// This struct is created by [`Catalog::viewer_preferences`].
+pub struct ViewerPreferences<'a> {
+    dict: Dict<'a>,
+}
+
+impl<'a> Writer<'a> for ViewerPreferences<'a> {
+    fn start(obj: Obj<'a>) -> Self {
+        Self { dict: obj.dict() }
+    }
+}
+
+impl<'a> ViewerPreferences<'a> {
+    /// Write the `/HideToolbar` attribute to set whether the viewer should hide
+    /// its toolbars while the document is open.
+    pub fn hide_toolbar(&mut self, hide: bool) -> &mut Self {
+        self.pair(Name(b"HideToolbar"), hide);
+        self
+    }
+
+    /// Write the `/HideMenubar` attribute to set whether the viewer should hide
+    /// its menu bar while the document is open.
+    pub fn hide_menubar(&mut self, hide: bool) -> &mut Self {
+        self.pair(Name(b"HideMenubar"), hide);
+        self
+    }
+
+    /// Write the `/FitWindow` attribute to set whether the viewer should resize
+    /// its window to the size of the first page.
+    pub fn fit_window(&mut self, fit: bool) -> &mut Self {
+        self.pair(Name(b"FitWindow"), fit);
+        self
+    }
+
+    /// Write the `/CenterWindow` attribute to set whether the viewer should
+    /// center its window on the screen.
+    pub fn center_window(&mut self, center: bool) -> &mut Self {
+        self.pair(Name(b"CenterWindow"), center);
+        self
+    }
+
+    /// Write the `/NonFullScreenPageMode` attribute to set which chrome
+    /// elements the viewer should show for a document which requests full
+    /// screen rendering in its catalog when it is not shown in full screen
+    /// mode.
+    ///
+    /// Panics if `mode` is [`PageMode::FullScreen`].
+    pub fn non_full_screen_page_mode(&mut self, mode: PageMode) -> &mut Self {
+        assert!(mode != PageMode::FullScreen, "mode must not full screen");
+        self.pair(Name(b"NonFullScreenPageMode"), mode.to_name());
+        self
+    }
+
+    /// Write the `/Direction` attribute to aid the viewer in how to lay out the
+    /// pages visually. PDF 1.3+.
+    pub fn direction(&mut self, dir: Direction) -> &mut Self {
+        self.pair(Name(b"Direction"), dir.to_name());
+        self
+    }
+}
+
+deref!('a, ViewerPreferences<'a> => Dict<'a>, dict);
+
+/// Predominant reading order of text.
+///
+/// Used to aid the viewer with the spacial ordering in which to display pages.
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Hash)]
+pub enum Direction {
+    /// Left-to-right.
+    L2R,
+    /// Right-to-left as well as vertical writing systems.
+    R2L,
+}
+
+impl Direction {
+    pub(crate) fn to_name(self) -> Name<'static> {
+        match self {
+            Self::L2R => Name(b"L2R"),
+            Self::R2L => Name(b"R2L"),
+        }
+    }
+}
+
+/// Writer for a _page label dictionary_.
+pub struct PageLabel<'a> {
+    dict: Dict<'a>,
+}
+
+impl<'a> Writer<'a> for PageLabel<'a> {
+    fn start(obj: Obj<'a>) -> Self {
+        let mut dict = obj.dict();
+        dict.pair(Name(b"Type"), Name(b"PageLabel"));
+        Self { dict }
+    }
+}
+
+impl<'a> PageLabel<'a> {
+    /// Write the `/S` attribute to set the page label's numbering style.
+    ///
+    /// If this attribute is omitted, only the prefix will be used, there will
+    /// be no page number.
+    pub fn style(&mut self, style: NumberingStyle) -> &mut Self {
+        self.pair(Name(b"S"), style.to_name());
+        self
+    }
+
+    /// Write the `/P` attribute to set the page label's prefix.
+    pub fn prefix(&mut self, prefix: TextStr) -> &mut Self {
+        self.pair(Name(b"P"), prefix);
+        self
+    }
+
+    /// Write the `/St` attribute to set the page label's offset.
+    ///
+    /// This must be greater or equal to `1` if set.
+    pub fn offset(&mut self, offset: i32) -> &mut Self {
+        self.pair(Name(b"St"), offset);
+        self
+    }
+}
+
+deref!('a, PageLabel<'a> => Dict<'a>, dict);
+
+/// The numbering style of a page label.
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Hash)]
+pub enum NumberingStyle {
+    /// Arabic numerals.
+    Arabic,
+    /// Lowercase Roman numerals.
+    LowerRoman,
+    /// Uppercase Roman numerals.
+    UpperRoman,
+    /// Lowercase letters (a-z, then aa-zz, ...).
+    LowerAlpha,
+    /// Uppercase letters (A-Z, then AA-ZZ, ...).
+    UpperAlpha,
+}
+
+impl NumberingStyle {
+    pub(crate) fn to_name(self) -> Name<'static> {
+        match self {
+            NumberingStyle::Arabic => Name(b"D"),
+            NumberingStyle::LowerRoman => Name(b"r"),
+            NumberingStyle::UpperRoman => Name(b"R"),
+            NumberingStyle::LowerAlpha => Name(b"a"),
+            NumberingStyle::UpperAlpha => Name(b"A"),
+        }
+    }
+}
+
+/// Writer for a _document information dictionary_.
+///
+/// This struct is created by [`PdfWriter::document_info`].
+pub struct DocumentInfo<'a> {
+    dict: Dict<'a>,
+}
+
+impl<'a> Writer<'a> for DocumentInfo<'a> {
+    fn start(obj: Obj<'a>) -> Self {
+        Self { dict: obj.dict() }
+    }
+}
+
+impl<'a> DocumentInfo<'a> {
+    /// Write the `/Title` attribute to set the document's title. PDF 1.1+.
+    pub fn title(&mut self, title: TextStr) -> &mut Self {
+        self.pair(Name(b"Title"), title);
+        self
+    }
+
+    /// Write the `/Author` attribute to set the document's author.
+    pub fn author(&mut self, author: TextStr) -> &mut Self {
+        self.pair(Name(b"Author"), author);
+        self
+    }
+
+    /// Write the `/Subject` attribute to set the document's subject. PDF 1.1+.
+    pub fn subject(&mut self, subject: TextStr) -> &mut Self {
+        self.pair(Name(b"Subject"), subject);
+        self
+    }
+
+    /// Write the `/Keywords` attribute to set terms associated to the document.
+    /// PDF 1.1+.
+    pub fn keywords(&mut self, keywords: TextStr) -> &mut Self {
+        self.pair(Name(b"Keywords"), keywords);
+        self
+    }
+
+    /// Write the `/Creator` attribute to set the name of the product that
+    /// converted or wrote the file that this PDF has been converted from.
+    pub fn creator(&mut self, creator: TextStr) -> &mut Self {
+        self.pair(Name(b"Creator"), creator);
+        self
+    }
+
+    /// Write the `/Producer` attribute to set the name of the product that
+    /// converted or wrote this PDF.
+    pub fn producer(&mut self, producer: TextStr) -> &mut Self {
+        self.pair(Name(b"Producer"), producer);
+        self
+    }
+
+    /// Write the `/CreationDate` attribute to set the date the document was
+    /// created.
+    pub fn creation_date(&mut self, date: Date) -> &mut Self {
+        self.pair(Name(b"CreationDate"), date);
+        self
+    }
+
+    /// Write the `/ModDate` attribute to set the date the document was last
+    /// modified.
+    ///
+    /// Required if `/PieceInfo` is set in the document catalog.
+    pub fn modified_date(&mut self, date: Date) -> &mut Self {
+        self.pair(Name(b"ModDate"), date);
+        self
+    }
+
+    /// Write the `/Trapped` attribute to set whether the document is fully or
+    /// partially trapped. PDF 1.3+.
+    pub fn trapped(&mut self, trapped: TrappingStatus) -> &mut Self {
+        self.pair(Name(b"Trapped"), trapped.to_name());
+        self
+    }
+}
+
+deref!('a, DocumentInfo<'a> => Dict<'a>, dict);
+
+/// Whether a document has been adjusted with traps to account for colorant
+/// misregistration during the printing process.
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Hash)]
+pub enum TrappingStatus {
+    /// The document is fully trapped.
+    Trapped,
+    /// The document has not been trapped.
+    NotTrapped,
+    /// The document is partially trapped or the trapping status is unknown.
+    Unknown,
+}
+
+impl TrappingStatus {
+    pub(crate) fn to_name(self) -> Name<'static> {
+        match self {
+            TrappingStatus::Trapped => Name(b"True"),
+            TrappingStatus::NotTrapped => Name(b"False"),
+            TrappingStatus::Unknown => Name(b"Unknown"),
+        }
+    }
+}
+
+/// Writer for a _page tree dictionary_.
+///
+/// This struct is created by [`PdfWriter::pages`].
+pub struct Pages<'a> {
+    dict: Dict<'a>,
+}
+
+impl<'a> Writer<'a> for Pages<'a> {
+    fn start(obj: Obj<'a>) -> Self {
+        let mut dict = obj.dict();
+        dict.pair(Name(b"Type"), Name(b"Pages"));
+        Self { dict }
+    }
+}
+
+impl<'a> Pages<'a> {
+    /// Write the `/Parent` attribute. Required except in root node.
+    pub fn parent(&mut self, parent: Ref) -> &mut Self {
+        self.pair(Name(b"Parent"), parent);
+        self
+    }
+
+    /// Write the `/Kids` attributes, listing the immediate children of this
+    /// node in the page tree. Required.
+    pub fn kids(&mut self, kids: impl IntoIterator<Item = Ref>) -> &mut Self {
+        self.insert(Name(b"Kids")).array().items(kids);
+        self
+    }
+
+    /// Write the `/Count` attribute, specifying how many descendants this node
+    /// in the page tree has. This may be different to the length of `/Kids`
+    /// when the tree has multiple layers. Required.
+    pub fn count(&mut self, count: i32) -> &mut Self {
+        self.pair(Name(b"Count"), count);
+        self
+    }
+
+    /// Write the `/MediaBox` attribute.
+    pub fn media_box(&mut self, rect: Rect) -> &mut Self {
+        self.pair(Name(b"MediaBox"), rect);
+        self
+    }
+
+    /// Start writing the `/Resources` dictionary.
+    pub fn resources(&mut self) -> Resources<'_> {
+        self.insert(Name(b"Resources")).start()
+    }
+}
+
+deref!('a, Pages<'a> => Dict<'a>, dict);
+
+/// Writer for a _page dictionary_.
+///
+/// This struct is created by [`PdfWriter::page`].
+pub struct Page<'a> {
+    dict: Dict<'a>,
+}
+
+impl<'a> Writer<'a> for Page<'a> {
+    fn start(obj: Obj<'a>) -> Self {
+        let mut dict = obj.dict();
+        dict.pair(Name(b"Type"), Name(b"Page"));
+        Self { dict }
+    }
+}
+
+impl<'a> Page<'a> {
+    /// Write the `/Parent` attribute. Required.
+    pub fn parent(&mut self, parent: Ref) -> &mut Self {
+        self.pair(Name(b"Parent"), parent);
+        self
+    }
+
+    /// Write the `/LastModified` attribute. PDF 1.3+.
+    pub fn last_modified(&mut self, date: Date) -> &mut Self {
+        self.pair(Name(b"LastModified"), date);
+        self
+    }
+
+    /// Write the `/MediaBox` attribute. This is the size of the physical medium
+    /// the page gets printed onto.
+    pub fn media_box(&mut self, rect: Rect) -> &mut Self {
+        self.pair(Name(b"MediaBox"), rect);
+        self
+    }
+
+    /// Write the `/CropBox` attribute. This is the size of the area within
+    /// which content is visible.
+    pub fn crop_box(&mut self, rect: Rect) -> &mut Self {
+        self.pair(Name(b"CropBox"), rect);
+        self
+    }
+
+    /// Write the `/BleedBox` attribute. This is the size of the area within
+    /// which content is visible in a print production environment. Most
+    /// production-aiding marks should be outside of this box. PDF 1.3+.
+    pub fn bleed_box(&mut self, rect: Rect) -> &mut Self {
+        self.pair(Name(b"BleedBox"), rect);
+        self
+    }
+
+    /// Write the `/TrimBox` attribute. This is the size of the produced
+    /// document after trimming is applied. PDF 1.3+.
+    pub fn trim_box(&mut self, rect: Rect) -> &mut Self {
+        self.pair(Name(b"TrimBox"), rect);
+        self
+    }
+
+    /// Write the `/ArtBox` attribute. This is the area that another program
+    /// importing this file should use. PDF 1.3+.
+    pub fn art_box(&mut self, rect: Rect) -> &mut Self {
+        self.pair(Name(b"ArtBox"), rect);
+        self
+    }
+
+    /// Start writing the `/Resources` dictionary.
+    pub fn resources(&mut self) -> Resources<'_> {
+        self.insert(Name(b"Resources")).start()
+    }
+
+    /// Write the `/Contents` attribute as reference to a single content stream.
+    ///
+    /// Such a content stream can be created using the [`Content`] builder and
+    /// written to the file using [`PdfWriter::stream`].
+    pub fn contents(&mut self, id: Ref) -> &mut Self {
+        self.pair(Name(b"Contents"), id);
+        self
+    }
+
+    /// Write the `/Contents` attribute as an array.
+    pub fn contents_array(&mut self, ids: impl IntoIterator<Item = Ref>) -> &mut Self {
+        self.insert(Name(b"Contents")).array().items(ids);
+        self
+    }
+
+    /// Write the `/Rotate` attribute. This is the number of degrees the page
+    /// should be rotated clockwise when displayed. This should be a multiple
+    /// of 90.
+    pub fn rotate(&mut self, degrees: i32) -> &mut Self {
+        self.pair(Name(b"Rotate"), degrees);
+        self
+    }
+
+    /// Start writing the `/Group` dictionary to set the transparency settings
+    /// for the page. PDF 1.4+.
+    pub fn group(&mut self) -> Group<'_> {
+        self.insert(Name(b"Group")).start()
+    }
+
+    /// Write the `/Dur` attribute. This is the amount of seconds the page
+    /// should be displayed before advancing to the next one. PDF 1.1+.
+    pub fn duration(&mut self, seconds: f32) -> &mut Self {
+        self.pair(Name(b"Dur"), seconds);
+        self
+    }
+
+    /// Start writing the `/Trans` dictionary. This sets a transition effect for
+    /// advancing to the next page. PDF 1.1+.
+    pub fn transition(&mut self) -> Transition<'_> {
+        self.insert(Name(b"Trans")).start()
+    }
+
+    /// Start writing the `/Annots` (annotations) array.
+    pub fn annotations(&mut self) -> Annotations<'_> {
+        self.insert(Name(b"Annots")).start()
+    }
+
+    /// Write the `/Tabs` attribute. This specifies the order in which the
+    /// annotations should be focussed by hitting tab. PDF 1.5+.
+    pub fn tab_order(&mut self, order: TabOrder) -> &mut Self {
+        self.pair(Name(b"Tabs"), order.to_name());
+        self
+    }
+}
+
+deref!('a, Page<'a> => Dict<'a>, dict);
+
 /// Writer for an _outline dictionary_.
 ///
 /// This struct is created by [`PdfWriter::outline`].
 pub struct Outline<'a> {
-    dict: Dict<'a, IndirectGuard>,
+    dict: Dict<'a>,
 }
 
-impl<'a> Outline<'a> {
-    pub(crate) fn start(obj: Obj<'a, IndirectGuard>) -> Self {
+impl<'a> Writer<'a> for Outline<'a> {
+    fn start(obj: Obj<'a>) -> Self {
         let mut dict = obj.dict();
         dict.pair(Name(b"Type"), Name(b"Outlines"));
         Self { dict }
     }
+}
 
+impl<'a> Outline<'a> {
     /// Write the `/First` attribute which points to the first
     /// [item](OutlineItem) in the document's outline.
     pub fn first(&mut self, item: Ref) -> &mut Self {
@@ -301,33 +592,32 @@ impl<'a> Outline<'a> {
     /// Write the `/Count` attribute. This tells the viewer how many outline
     /// elements (at all levels) are currently visible.
     ///
-    /// Panics the `items` is negative.
+    /// Panics if `count` is negative.
     pub fn count(&mut self, count: i32) -> &mut Self {
-        if count < 0 {
-            panic!("count must not be negative");
-        }
-
+        assert!(count >= 0, "visible outline count must not be negative");
         self.pair(Name(b"Count"), count);
         self
     }
 }
 
-deref!('a, Outline<'a> => Dict<'a, IndirectGuard>, dict);
+deref!('a, Outline<'a> => Dict<'a>, dict);
 
 /// Writer for an _outline item dictionary_.
 ///
 /// This struct is created by [`PdfWriter::outline_item`].
 pub struct OutlineItem<'a> {
-    dict: Dict<'a, IndirectGuard>,
+    dict: Dict<'a>,
 }
 
-impl<'a> OutlineItem<'a> {
-    pub(crate) fn start(obj: Obj<'a, IndirectGuard>) -> Self {
+impl<'a> Writer<'a> for OutlineItem<'a> {
+    fn start(obj: Obj<'a>) -> Self {
         let mut dict = obj.dict();
         dict.pair(Name(b"Type"), Name(b"Outlines"));
         Self { dict }
     }
+}
 
+impl<'a> OutlineItem<'a> {
     /// Write the `/Title` attribute.
     pub fn title(&mut self, title: TextStr) -> &mut Self {
         self.pair(Name(b"Title"), title);
@@ -378,8 +668,8 @@ impl<'a> OutlineItem<'a> {
 
     /// Start writing the `/Dest` attribute to set the destination of this
     /// outline item.
-    pub fn dest_direct(&mut self, page: Ref) -> Destination<'_> {
-        Destination::start(self.key(Name(b"Dest")), page)
+    pub fn dest_direct(&mut self) -> Destination<'_> {
+        self.insert(Name(b"Dest")).start()
     }
 
     /// Write the `/Dest` attribute to set the destination of this
@@ -390,20 +680,20 @@ impl<'a> OutlineItem<'a> {
     }
 
     /// Write the `/C` attribute using an RGB color. This sets the color in
-    /// which the outline item's title should be rendered. (1.4+)
+    /// which the outline item's title should be rendered. PDF 1.4+.
     pub fn color_rgb(&mut self, r: f32, g: f32, b: f32) -> &mut Self {
-        self.key(Name(b"C")).array().typed().items([r, g, b]);
+        self.insert(Name(b"C")).array().items([r, g, b]);
         self
     }
 
-    /// Write the `/F` attribute. (1.4+)
+    /// Write the `/F` attribute. PDF 1.4+.
     pub fn flags(&mut self, flags: OutlineItemFlags) -> &mut Self {
         self.pair(Name(b"F"), flags.bits() as i32);
         self
     }
 }
 
-deref!('a, OutlineItem<'a> => Dict<'a, IndirectGuard>, dict);
+deref!('a, OutlineItem<'a> => Dict<'a>, dict);
 
 bitflags::bitflags! {
     /// Bitflags describing the appearance of an outline item.
@@ -419,34 +709,43 @@ bitflags::bitflags! {
 ///
 /// This struct is created by [`PdfWriter::destinations`].
 pub struct Destinations<'a> {
-    dict: Dict<'a, IndirectGuard>,
+    dict: Dict<'a>,
+}
+
+impl<'a> Writer<'a> for Destinations<'a> {
+    fn start(obj: Obj<'a>) -> Self {
+        Self { dict: obj.dict() }
+    }
 }
 
 impl<'a> Destinations<'a> {
-    pub(crate) fn start(obj: Obj<'a, IndirectGuard>) -> Self {
-        Self { dict: obj.dict() }
-    }
-
     /// Start adding another named destination.
-    pub fn insert(&mut self, name: Name, page: Ref) -> Destination<'_> {
-        Destination::start(self.key(name), page)
+    pub fn insert(&mut self, name: Name) -> Destination<'_> {
+        self.dict.insert(name).start()
     }
 }
 
-deref!('a, Destinations<'a> => Dict<'a, IndirectGuard>, dict);
+deref!('a, Destinations<'a> => Dict<'a>, dict);
 
 /// Writer for a _destination array_.
 ///
-/// This struct is created by [`Destinations::insert`] and [`Action::dest_direct`].
+/// This struct is created by [`Destinations::insert`] and
+/// [`Action::destination_direct`].
 pub struct Destination<'a> {
     array: Array<'a>,
 }
 
+impl<'a> Writer<'a> for Destination<'a> {
+    fn start(obj: Obj<'a>) -> Self {
+        Self { array: obj.array() }
+    }
+}
+
 impl<'a> Destination<'a> {
-    pub(crate) fn start(obj: Obj<'a>, page: Ref) -> Self {
-        let mut array = obj.array();
-        array.item(page);
-        Self { array }
+    /// The target page. Required.
+    pub fn page(mut self, page: Ref) -> Self {
+        self.item(page);
+        self
     }
 
     /// Write the `/XYZ` command which skips to the specified coordinated.
@@ -481,27 +780,24 @@ impl<'a> Destination<'a> {
     /// screen.
     pub fn fit_rect(mut self, rect: Rect) {
         self.item(Name(b"FitR"));
-        self.item(rect.x1);
-        self.item(rect.y1);
-        self.item(rect.x2);
-        self.item(rect.y2);
+        self.array.items([rect.x1, rect.y1, rect.x2, rect.y2]);
     }
 
     /// Write the `/FitB` command which fits all of the referenced page's
-    /// content on screen. (1.1+)
+    /// content on screen. PDF 1.1+.
     pub fn fit_bounding_box(mut self) {
         self.item(Name(b"FitB"));
     }
 
     /// Write the `/FitBH` command which fits the referenced page's content to
-    /// the screen width and skips to the specified offset. (1.1+)
+    /// the screen width and skips to the specified offset. PDF 1.1+.
     pub fn fit_bounding_box_horizontal(mut self, top: f32) {
         self.item(Name(b"FitBH"));
         self.item(top);
     }
 
     /// Write the `/FitBV` command which fits the referenced page's content to
-    /// the screen height and skips to the specified offset. (1.1+)
+    /// the screen height and skips to the specified offset. PDF 1.1+.
     pub fn fit_bounding_box_vertical(mut self, left: f32) {
         self.item(Name(b"FitBV"));
         self.item(left);
@@ -510,87 +806,21 @@ impl<'a> Destination<'a> {
 
 deref!('a, Destination<'a> => Array<'a>, array);
 
-/// Writer for a _viewer preference dictionary_.
-///
-/// This struct is created by [`Catalog::viewer_preferences`].
-pub struct ViewerPreferences<'a> {
-    dict: Dict<'a>,
-}
-
-impl<'a> ViewerPreferences<'a> {
-    pub(crate) fn new(obj: Obj<'a>) -> Self {
-        Self { dict: obj.dict() }
-    }
-
-    /// Write the `/HideToolbar` attribute to set whether the viewer should hide
-    /// its toolbars while the document is open.
-    pub fn hide_toolbar(&mut self, hide: bool) -> &mut Self {
-        self.pair(Name(b"HideToolbar"), hide);
-        self
-    }
-
-    /// Write the `/HideMenubar` attribute to set whether the viewer should hide
-    /// its menu bar while the document is open.
-    pub fn hide_menubar(&mut self, hide: bool) -> &mut Self {
-        self.pair(Name(b"HideMenubar"), hide);
-        self
-    }
-
-    /// Write the `/FitWindow` attribute to set whether the viewer should resize
-    /// its window to the size of the first page.
-    pub fn fit_window(&mut self, fit: bool) -> &mut Self {
-        self.pair(Name(b"FitWindow"), fit);
-        self
-    }
-
-    /// Write the `/CenterWindow` attribute to set whether the viewer should
-    /// center its window on the screen.
-    pub fn center_window(&mut self, center: bool) -> &mut Self {
-        self.pair(Name(b"CenterWindow"), center);
-        self
-    }
-
-    /// Write the `/NonFullScreenPageMode` attribute to set which chrome
-    /// elements the viewer should show for a document which requests full
-    /// screen rendering in its catalog when it is not shown in full screen
-    /// mode.
-    ///
-    /// Panics if the mode is [`PageMode::FullScreen`].
-    pub fn non_full_screen_page_mode(&mut self, mode: PageMode) -> &mut Self {
-        if mode == PageMode::FullScreen {
-            panic!("mode must not full screen");
-        }
-
-        self.pair(Name(b"NonFullScreenPageMode"), mode.to_name());
-        self
-    }
-
-    /// Write the `/Direction` attribute to aid the viewer in how to lay out the
-    /// pages visually. (1.3+)
-    pub fn direction(&mut self, dir: Direction) -> &mut Self {
-        self.pair(Name(b"Direction"), dir.to_name());
-        self
-    }
-}
-
-deref!('a, ViewerPreferences<'a> => Dict<'a>, dict);
-
-/// Predominant reading order of text.
-///
-/// Used to aid the viewer with the spacial ordering in which to display pages.
+/// What order to tab through the annotations on a page.
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Hash)]
-pub enum Direction {
-    /// Left-to-right.
-    L2R,
-    /// Right-to-left as well as vertical writing systems.
-    R2L,
+#[allow(missing_docs)]
+pub enum TabOrder {
+    RowOrder,
+    ColumnOrder,
+    StructureOrder,
 }
 
-impl Direction {
-    fn to_name(self) -> Name<'static> {
+impl TabOrder {
+    pub(crate) fn to_name(self) -> Name<'static> {
         match self {
-            Self::L2R => Name(b"L2R"),
-            Self::R2L => Name(b"R2L"),
+            Self::RowOrder => Name(b"R"),
+            Self::ColumnOrder => Name(b"C"),
+            Self::StructureOrder => Name(b"S"),
         }
     }
 }
