@@ -7,10 +7,12 @@ macro_rules! test {
         let mut expected = vec![];
         $(expected.extend($expected);)*
         if buf != expected {
-            assert_eq!(
-                String::from_utf8_lossy(&buf),
-                String::from_utf8_lossy(&expected),
-            );
+            println!("=========== EXPECTED =============");
+            println!("{}", String::from_utf8_lossy(&expected));
+            println!("============= FOUND ==============");
+            println!("{}", String::from_utf8_lossy(&buf));
+            println!("=============================");
+            panic!("assertion failed");
         }
     }}
 }
@@ -111,6 +113,18 @@ fn test_xref_free_list_long() {
     )
 }
 
+#[test]
+#[should_panic(expected = "duplicate indirect reference id: 3")]
+fn test_xref_free_list_duplicate() {
+    let mut w = PdfWriter::new();
+    w.indirect(Ref::new(3)).primitive(1);
+    w.indirect(Ref::new(5)).primitive(2);
+    w.indirect(Ref::new(13)).primitive(1);
+    w.indirect(Ref::new(3)).primitive(1);
+    w.indirect(Ref::new(6)).primitive(2);
+    w.finish();
+}
+
 #[test]
 fn test_primitive_objects() {
     // Test really simple objects.
@@ -135,7 +149,7 @@ fn test_primitive_objects() {
     // Test names.
     test_primitive!(Name(b"Filter"), b"/Filter");
     test_primitive!(Name(b"A B"), br"/A#20B");
-    test_primitive!(Name(b"~+c"), br"/~+c");
+    test_primitive!(Name(b"~+c"), br"/#7E#2Bc");
     test_primitive!(Name(b"#"), br"/#23");
     test_primitive!(Name(b"\n"), br"/#0A");
 }
